@@ -8,10 +8,14 @@ use Pod::Usage qw( pod2usage );
 use Clustericious::Config 1.03;
 use Term::ANSIColor ();
 use Sys::Hostname qw( hostname );
+use YAML::XS qw( Dump );
+use JSON::MaybeXS qw( encode_json );
+use File::Basename qw( basename );
 use AE;
+use Clustericious::Admin::RemoteHandler;
 
 # ABSTRACT: Parallel SSH client
-our $VERSION = '1.01'; # VERSION
+our $VERSION = '1.02'; # VERSION
 
 
 sub main
@@ -220,6 +224,73 @@ sub next_color
   $colors[ ++$self->{next_color} ] // $colors[ $self->{next_color} = 0 ];
 }
 
+sub payload
+{
+  my($self, $clustername) = @_;
+  
+  my %env = $self->config->env( default => {} );
+  $env{CLUSTER}      //= $clustername; # deprecate
+  $env{CLAD_CLUSTER} //= $clustername;
+
+  my $payload = {
+    env     => \%env,
+    command => $self->command,
+    verbose => $self->verbose,
+    version => $App::clad::VERSION // 'dev',
+  };
+  
+  if($self->files)
+  {
+    $payload->{require} = '1.01';
+    
+    foreach my $filename ($self->files)
+    {
+      my %h;
+      open my $fh, '<', $filename;
+      binmode $fh;
+      $h{content} = do { local $/; <$fh> };
+      close $fh;
+      $h{name} = basename $filename;
+      $h{mode} = (stat "/etc/passwd")[2] & 0777;
+      push @{ $payload->{files} }, \%h;
+    }
+  }
+  
+  if($self->script)
+  {
+    my($name, $content) = $self->script;
+    $payload->{require} = '1.01';
+    
+    push @{ $payload->{files} }, {
+      name    => $name,
+      content => $content,
+      mode    => '0700',
+      env     => 'SCRIPT1',
+    };
+  }
+  
+  if($self->fat)
+  {
+    # Perl on the remote end may not have YAML
+    # (actually it may not have JSON::PP either
+    # but at least that is part of the core as
+    # of 5.14).
+    $payload = encode_json($payload);
+    require Clustericious::Admin::Server;
+    open my $fh, '<', $INC{'Clustericious/Admin/Server.pm'};
+    my $code = do { local $/; <$fh> };
+    close $fh;
+    $code =~ s{\s*$}{"\n"}e;
+    $payload = $code . $payload;
+  }
+  else
+  {
+    $payload = Dump($payload);
+  }
+  
+  $payload;
+}
+
 sub run
 {
   my($self) = @_;
@@ -235,9 +306,7 @@ sub run
   {
     my $user = $cluster =~ s/^(.*)@// ? $1 : $self->user;
 
-    my %env = $self->config->env( default => {} );
-    $env{CLUSTER}      //= $cluster; # deprecate
-    $env{CLAD_CLUSTER} //= $cluster;
+    my $payload = $self->payload($cluster);
 
     foreach my $host (@{ $self->cluster_list->{$cluster} })
     {
@@ -249,11 +318,11 @@ sub run
       else
       {
         my $remote = Clustericious::Admin::RemoteHandler->new(
-          prefix => $prefix,
-          clad   => $self,
-          env    => \%env,
-          user   => $user,
-          host   => $host,
+          prefix  => $prefix,
+          clad    => $self,
+          user    => $user,
+          host    => $host,
+          payload => $payload,
         );
 
         my $done = $remote->cv;
@@ -290,160 +359,6 @@ sub run_server
   Clustericious::Admin::Server->_server(*STDIN);
 }
 
-package Clustericious::Admin::RemoteHandler;
-
-use strict;
-use warnings;
-use AE;
-use AnyEvent::Open3::Simple 0.76;
-use YAML::XS qw( Dump );
-use JSON::MaybeXS qw( encode_json );
-use File::Basename qw( basename );
-
-our $VERSION = '1.01'; # VERSION
-
-sub new
-{
-  my($class, %args) = @_;
-  
-  # args: prefix, clad, env, user, host
-  
-  my $self = bless {
-    prefix => $args{prefix},
-    clad   => $args{clad},
-    cv     => AE::cv,
-  }, $class;
-  
-  my $clad = $args{clad};
-  
-  my $done = $self->{cv};
-  
-  my $ipc = AnyEvent::Open3::Simple->new(
-    on_start => sub {
-      my($proc, $program, @args) = @_;
-      $self->print_line(star => "% $program @args") if $clad->verbose;
-    },
-    on_stdout => sub {
-      my($proc, $line) = @_;
-      $self->print_line(out => $line);
-    },
-    on_stderr => sub {
-      my($proc, $line) = @_;
-      $self->print_line(err => $line);
-    },
-    on_exit => sub {
-      my($proc, $exit, $signal) = @_;
-      $self->print_line(exit => $exit) if $exit;
-      $self->print_line(sig  => $signal) if $signal;
-      $clad->ret(2) if $exit || $signal;
-      $done->send;
-    },
-    on_error => sub {
-      my($error) = @_;
-      $self->print_line(fail => $error);
-      $clad->ret(2);
-      $done->send;
-    },
-  );
-  
-  # can/should we generate this payload once, instead
-  # of for each host?
-  my $payload = {
-    env     => $args{env},
-    command => $clad->command,
-    verbose => $clad->verbose,
-    version => $App::clad::VERSION // 'dev',
-  };
-  
-  if($self->clad->files)
-  {
-    $payload->{require} = '1.01';
-    
-    foreach my $filename ($self->clad->files)
-    {
-      my %h;
-      open my $fh, '<', $filename;
-      binmode $fh;
-      $h{content} = do { local $/; <$fh> };
-      close $fh;
-      $h{name} = basename $filename;
-      $h{mode} = (stat "/etc/passwd")[2] & 0777;
-      push @{ $payload->{files} }, \%h;
-    }
-  }
-  
-  if($self->clad->script)
-  {
-    my($name, $content) = $self->clad->script;
-    $payload->{require} = '1.01';
-    
-    push @{ $payload->{files} }, {
-      name    => $name,
-      content => $content,
-      mode    => '0700',
-      env     => 'SCRIPT1',
-    };
-  }
-  
-  if($self->clad->fat)
-  {
-    # Perl on the remote end may not have YAML
-    # (actually it may not have JSON::PP either
-    # but at least that is part of the core as
-    # of 5.14).
-    $payload = encode_json($payload);
-    require Clustericious::Admin::Server;
-    open my $fh, '<', $INC{'Clustericious/Admin/Server.pm'};
-    my $code = do { local $/; <$fh> };
-    close $fh;
-    $code =~ s{\s*$}{"\n"}e;
-    $payload = $code . $payload;
-  }
-  else
-  {
-    $payload = Dump($payload);
-  }
-  
-  $ipc->run(
-    $clad->ssh_command,
-    $clad->ssh_options,
-    $clad->ssh_extra,
-    ($args{user} ? ('-l' => $args{user}) : ()),
-    $args{host},
-    $clad->server_command,
-    \$payload,
-  );
-  
-  $self;
-}
-
-sub clad   { shift->{clad} }
-sub prefix { shift->{prefix} }
-
-sub color
-{
-  my($self) = @_;
-  $self->{color} //= $self->clad->next_color;
-}
-
-sub is_color
-{
-  my($self) = @_;
-  $self->{is_color} //= $self->clad->color;
-}
-
-sub print_line
-{
-  my($self, $code, $line) = @_;
-  
-  print Term::ANSIColor::color($self->color) if $self->is_color;
-  printf "[%@{[ $self->clad->host_length ]}s %-4s] ", $self->prefix, $code;
-  print Term::ANSIColor::color('reset') if $self->is_color;
-  print $line, "\n";
-}
-
-sub cv { shift->{cv} }
-
 1;
 
 __END__
@@ -458,7 +373,7 @@ App::clad - Parallel SSH client
 
 =head1 VERSION
 
-version 1.01
+version 1.02
 
 =head1 SYNOPSIS
 
