@@ -5,9 +5,10 @@ use warnings;
 use Sys::Hostname qw( hostname );
 use File::Temp qw( tempdir );
 use File::Spec;
+use File::Path qw( mkpath );
 
 # ABSTRACT: Parallel SSH client server side code
-our $VERSION = '1.01'; # VERSION
+our $VERSION = '1.02'; # VERSION
 
 
 # This is the implementation of the clad server.
@@ -94,12 +95,19 @@ sub _server
   #     server should die if requirement isn't met
   #     ignored if set to 'dev'
   #
-  #   files: optional list of hashref
+  #   files: optional list of hashref   [ 1.01 ]
   #     each hashref has:
   #       name: the file basename (no directory)
   #       content: the content of the file
   #       mode: (optional) octal unix permission mode as a string (ie "0755" or "0644")
   #       env: (optional) environment variable to use instead of FILEx
+  #
+  #   dir: optional hash of hash        [ 1.02 ]
+  #     each key is a path
+  #       each value is a hash
+  #         is_dir
+  #         content
+  #         mode
 
   if(ref $payload->{command} ne 'ARRAY' || @{ $payload->{command} } == 0)
   {
@@ -135,15 +143,41 @@ sub _server
     {
       my $path = File::Spec->catfile( tempdir( CLEANUP => 1 ), $file->{name} );
       open my $fh, '>', $path;
+      chmod oct($file->{mode}), $fh if defined $file->{mode};
       binmode $fh;
       print $fh $file->{content};
       close $fh;
-      chmod oct($file->{mode}), $path if defined $file->{mode};
       my $env = $file->{env};
       $env = "FILE@{[ $count++ ]}" unless defined $env;
       $ENV{$env} = $path;
     }
   }
+  
+  if($payload->{dir})
+  {
+    my $root = $ENV{DIR} = tempdir( CLEANUP => 1 );
+    
+    foreach my $name (sort keys %{ $payload->{dir} })
+    {
+      my $dir = $payload->{dir}->{$name};
+      next unless $dir->{is_dir};
+      my $path = File::Spec->catdir($root, $name);
+      mkdir $path;
+      chmod oct($dir->{mode}), $path if defined $dir->{mode};
+    }
+    
+    foreach my $name (sort keys %{ $payload->{dir} })
+    {
+      my $file = $payload->{dir}->{$name};
+      next if $file->{is_dir};
+      my $path = File::Spec->catfile($root, $name);
+      open my $fh, '>', $path;
+      chmod oct($file->{mode}), $fh if defined $file->{mode};
+      binmode $fh;
+      print $fh $file->{content};
+      close $fh;
+    }
+  }
 
   $ENV{$_} = $payload->{env}->{$_} for keys %{ $payload->{env} };
   
@@ -177,7 +211,7 @@ Clustericious::Admin::Server - Parallel SSH client server side code
 
 =head1 VERSION
 
-version 1.01
+version 1.02
 
 =head1 SYNOPSIS
 
